'use strict';

var view = require('@codemirror/view');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');

// Abyss theme colors
const base00 = '#000c18', // Background
base01 = '#6688cc', // Foreground
base02 = '#770811', // Selection
base03 = '#181f2f', // Dropdown Background
base05 = '#ddbb88', // Cursor
base06 = '#0055ff30', // Active line
base07 = '#FFFFFF', // Another placeholder
base08 = '#225588', // Keyword, Storage
base09 = '#2277ff', // Variable, Parameter
base0A = '#384887', // Comment
base0B = '#22aa44', // String, RegExp
base0C = '#9966b8', // Function, Type
base0D = '#f280d0', // Constant, Number
base0E = '#ddbb88', // Class
base0F = '#6688cc', // Heading
base10 = '#225588', // Tag
base11 = '#ff00ff8a', // Link
invalid = '#A22D44'; // Invalid
// Define the editor theme styles for Abyss.
const abyssTheme = view.EditorView.theme({
    '&': {
        color: base01,
        backgroundColor: base00
    },
    '.cm-content': {
        caretColor: base05
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: base05
    },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: base02
    },
    '.cm-searchMatch': {
        backgroundColor: base0C,
        outline: `1px solid ${base03}`,
        color: base07
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base11,
        color: base07
    },
    '.cm-panels': {
        backgroundColor: base03,
        color: base01
    },
    '.cm-activeLine': {
        backgroundColor: base06
    },
    '.cm-gutters': {
        backgroundColor: base03, // Assuming the dropdown background color for gutters
        color: base01
    },
    '.cm-tooltip': {
        backgroundColor: base01
    }
}, { dark: true });
// Define the highlighting style for code in the Abyss theme.
const abyssHighlightStyle = language.HighlightStyle.define([
    { tag: highlight.tags.keyword, color: base08, fontWeight: 'bold' },
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName], color: base09 },
    { tag: [highlight.tags.propertyName], color: base0C },
    { tag: [highlight.tags.processingInstruction, highlight.tags.string, highlight.tags.inserted, highlight.tags.special(highlight.tags.string)], color: base0B },
    { tag: [highlight.tags.function(highlight.tags.variableName), highlight.tags.labelName], color: base0C },
    { tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)], color: base0D },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: base09 },
    { tag: [highlight.tags.className], color: base0E },
    { tag: [highlight.tags.number, highlight.tags.changed, highlight.tags.annotation, highlight.tags.modifier, highlight.tags.self, highlight.tags.namespace], color: base0D },
    { tag: [highlight.tags.typeName], color: base0C },
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: base08 },
    { tag: [highlight.tags.url, highlight.tags.escape, highlight.tags.regexp, highlight.tags.link], color: base0B },
    { tag: [highlight.tags.meta, highlight.tags.comment], color: base0A },
    { tag: highlight.tags.tagName, color: base10 },
    { tag: highlight.tags.strong, fontWeight: 'bold' },
    { tag: highlight.tags.emphasis, fontStyle: 'italic' },
    { tag: highlight.tags.link, textDecoration: 'underline' },
    { tag: highlight.tags.heading, fontWeight: 'bold', color: base0F },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: base09 },
    { tag: highlight.tags.invalid, color: invalid, textDecoration: 'underline' }
]);
// Extension to enable the Abyss theme (both the editor theme and the highlight style).
const abyss = [
    abyssTheme,
    language.syntaxHighlighting(abyssHighlightStyle)
];

exports.abyss = abyss;
exports.abyssHighlightStyle = abyssHighlightStyle;
exports.abyssTheme = abyssTheme;
